@@ -52,21 +52,21 @@ zathura_db_save_jumplist(zathura_database_t* db, const char* file, girara_list_t
 }
 
 bool
-zathura_db_set_fileinfo(zathura_database_t* db, const char* file,
+zathura_db_set_fileinfo(zathura_database_t* db, const char* file, const uint8_t* hash_sha256,
                         zathura_fileinfo_t* file_info)
 {
-  g_return_val_if_fail(ZATHURA_IS_DATABASE(db) && file != NULL && file_info != NULL, false);
+  g_return_val_if_fail(ZATHURA_IS_DATABASE(db) && file != NULL && hash_sha256 != NULL && file_info != NULL, false);
 
-  return ZATHURA_DATABASE_GET_INTERFACE(db)->set_fileinfo(db, file, file_info);
+  return ZATHURA_DATABASE_GET_INTERFACE(db)->set_fileinfo(db, file, hash_sha256, file_info);
 }
 
 bool
-zathura_db_get_fileinfo(zathura_database_t* db, const char* file,
+zathura_db_get_fileinfo(zathura_database_t* db, const char* file, const uint8_t* hash_sha256,
                         zathura_fileinfo_t* file_info)
 {
-  g_return_val_if_fail(ZATHURA_IS_DATABASE(db) && file != NULL && file_info != NULL, false);
+  g_return_val_if_fail(ZATHURA_IS_DATABASE(db) && file != NULL && hash_sha256 != NULL && file_info != NULL, false);
 
-  return ZATHURA_DATABASE_GET_INTERFACE(db)->get_fileinfo(db, file, file_info);
+  return ZATHURA_DATABASE_GET_INTERFACE(db)->get_fileinfo(db, file, hash_sha256, file_info);
 }
 
 girara_list_t*
